@@ -1,34 +1,11 @@
 import ipywidgets as widgets
 from IPython.display import display, HTML, Javascript
 import os
+import pathlib
 from .custom_widgets import *
 from .auxfuncs import *
 
-
-#=====================================================================
-def load_stylesheets():
-    """
-    Load custom stylesheets for the widgets.
-    :return: A list of HTML elements containing the stylesheets.
-    """
-    # Get the directory of this file (core.py)
-    module_dir = os.path.dirname(os.path.abspath(__file__))
-    stylesheets = [
-        'assets/custom_widgets.css',
-        'assets/ipyformkit.css'
-    ]
-
-    sheets = []
-
-    for stylesheet in stylesheets:
-        stylesheet = module_dir + os.sep + stylesheet
-        if os.path.exists(stylesheet):
-            with open(stylesheet, 'r') as f:
-                css = f.read()
-                sheets.append(HTML(f'<style>{css}</style>'))
-        else:
-            print(f"Warning: {stylesheet} not found. Custom styles will not be applied.")
-    return sheets
+module_dir = os.path.dirname(os.path.abspath(__file__))
 
 #=====================================================================
 def create_widget(key, value):
@@ -44,8 +21,9 @@ def create_widget(key, value):
     box.add_class('ifk-widget-box')
     box.add_class('widget-vbox')
     
-    if value is None:
+    if callable(value):
         wid = widgets.Button(description=key)
+        wid.on_click(value)
         box.children = box.children[1:]
     elif isinstance(value, range):
         wid = widgets.IntSlider(value=value.start, min=value.start, max=value.stop, step=value.step)
@@ -56,21 +34,20 @@ def create_widget(key, value):
     elif isinstance(value, float):
         step = 10**(-count_decimal_places(value))
         wid = widgets.FloatText(value=value, step=step)
+    elif isinstance(value, pathlib.Path):
+        wid = FileAutocomplete(placeholder=str(value))
+        box.add_class('ifk-widget-FileAutocomplete')
     elif isinstance(value, str):
-        if os.sep in value:
-            wid = FileAutocomplete(placeholder=value)
-            box.add_class('ifk-widget-FileAutocomplete')
-        elif 'password' in key.lower():
+        if 'password' in key.lower():
             wid = widgets.Password(placeholder=value)
         elif value.endswith('...'):
             wid = widgets.Textarea(placeholder=value[:-3])
         else:
             wid = widgets.Text(placeholder=value)
     elif isinstance(value, tuple):
-        if value:
-            wid = widgets.Dropdown(options=value, value=value[0])
-        else:
-            wid = widgets.Label(value="(Empty list - no options)")
+        wid = widgets.Dropdown(options=value, value=value[0] if value else None)
+    elif isinstance(value, list):
+        wid = widgets.Select(options=value, value=value[0] if value else None, rows=min(10, len(value)))
     else:
         wid = widgets.Label(value=f"Unsupported type: {type(value).__name__}")
     
@@ -86,7 +63,7 @@ def create_widget(key, value):
     return box
 
 #=====================================================================
-def dict_to_form(input_dict, title=None, collapsed=None, nested=False):
+def dict_to_form(vbox, input_dict, title=None, collapsed=None, nested=False):
     """
     Convert a dictionary to a form with widgets.
 
@@ -131,7 +108,7 @@ def dict_to_form(input_dict, title=None, collapsed=None, nested=False):
 
         elif isinstance(value, dict):
             # Case: nested dictionary group
-            sub_vbox, sub_widgets_dict = dict_to_form(value, title=key, collapsed=True, nested=True)
+            sub_vbox, sub_widgets_dict = dict_to_form(vbox, value, title=key, collapsed=True, nested=True)
             widgets_dict.update(sub_widgets_dict)
             hbox_items.append(sub_vbox)
             
@@ -150,14 +127,12 @@ def dict_to_form(input_dict, title=None, collapsed=None, nested=False):
         vbox.label.add_class('ifk-widget-label')
         vbox.layout.width = '100%'
     else:
-        vbox = widgets.VBox(widgets_list)
-
-    if not nested: vbox.add_class('ifk-form')
+        vbox.children = widgets_list
 
     return vbox, widgets_dict
 
 #=====================================================================
-class Form(object):
+class Form(widgets.VBox):
     def __init__(self, input_dict, title=None, collapsed=None, max_width=600,
                  mandatory=None, disable=None, hide=None, check=None, tooltips=None):
         """
@@ -190,18 +165,18 @@ class Form(object):
 
         Attributes
         ----------
-        vbox : ipywidgets.VBox
-            The main container widget for the form.
         widgets_dict : dict
             A dictionary mapping field names to their corresponding widgets.
         """
+        super().__init__()
         
         self.title = title
         self._input_dict = input_dict
         self._mandatory = mandatory
 
-        self.vbox, self.widgets_dict = dict_to_form(input_dict, title=title, collapsed=collapsed)
-        self.vbox.layout.max_width = f'{max_width}px'
+        self.vbox, self.widgets_dict = dict_to_form(self, input_dict, title=title, collapsed=collapsed)
+        self.layout.max_width = f'{max_width}px'
+        self.add_class('ifk-form')
 
         if isinstance(mandatory, list):
             for key in mandatory:
@@ -218,6 +193,8 @@ class Form(object):
         self.update_hide()
 
         self.set_tooltips(tooltips)
+        self.load_stylesheets()
+        self.load_tooltip_script()
 
     #=====================================================================
     def add_observer(self, conditions, func):
@@ -328,12 +305,35 @@ class Form(object):
 
 
     #=====================================================================
-    def display(self):
+    def load_stylesheets(self):
         """
-        Display the form in the notebook.
+        Load custom stylesheets for the widgets.
+        :return: A list of HTML elements containing the stylesheets.
         """
-        items = [self.vbox, *load_stylesheets()]
-        display(*items)
+        # Get the directory of this file (core.py)
+        stylesheets = [
+            'assets/custom_widgets.css',
+            'assets/ipyformkit.css'
+        ]
+
+        for stylesheet in stylesheets:
+            stylesheet = StyleSheet(module_dir + os.sep + stylesheet)
+            self.children = self.children + (stylesheet,)
+
+    #=====================================================================
+    def load_tooltip_script(self):
+        hidden_output = widgets.Output(layout={'display': 'none'})
+
+        with hidden_output:
+            script = module_dir + os.sep + 'assets/tooltip_script.js'
+            if os.path.exists(script):
+                with open(script, 'r') as f:
+                    js = f.read()
+                    display(Javascript(js))
+            else:
+                print(f"Warning: {script} not found. Tooltips will not be functional.")
+                display(Javascript('console.warn("Tooltip script not found. Tooltips will not be functional.")'))
+
 
     #=====================================================================
     def get_values(self):
@@ -344,6 +344,7 @@ class Form(object):
         out = {key: wid.wid.value for key, wid in self.widgets_dict.items() if hasattr(wid.wid, 'value')}
         return out
     
+
     #=====================================================================
     def set_values(self, values, verbose=True):
         """
@@ -362,8 +363,7 @@ class Form(object):
                         wid.wid.value = value
                     else:
                         print(f"Warning: Type mismatch for {key}. Expected {type(wid.wid.value)}, got {type(value)}.")
-                elif value:
-                    print(f"Warning: {key} is not a valid widget.")
+
             elif verbose:
                 print(f"Warning: {key} is not a valid key in the form.")
 
@@ -377,8 +377,9 @@ class Form(object):
             else:
                 set_key(key, value)
     
+
     #=====================================================================
-    def check_and_return_values(self):
+    def check_and_return_values(self, verbose=True, disabled_ok=False, hidden_ok=False):
         """
         Check the values of the widgets in the form and return them as a dictionary.
         :return: A dictionary of key-value pairs representing the widget values, or None if validation fails.
@@ -396,27 +397,51 @@ class Form(object):
                 valid = check_func(value_dict)
                 
                 if value == '' and mandatory:
-                    print(f"Mandatory field '{key}' is empty.")
+                    if verbose: print(f"Mandatory field '{key}' is empty.")
                     wid.wid.add_class('ifk-widget-input-missing')
                     abort = True
                 else:
                     wid.wid.remove_class('ifk-widget-input-missing')
                     
                 if not valid:
-                    print(f"Invalid input in field '{key}'.")
+                    if verbose: print(f"Invalid input in field '{key}'.")
                     abort = True
                 
-                if not (disabled or hidden):
+                ok = not disabled or disabled_ok
+                ok = ok and (not hidden or hidden_ok)
+                
+                if ok:
                     out[key] = value
                     
         if abort:
             return None
         else:
             return out
+        
+
+    #=====================================================================
+    def __getitem__(self, key):
+        """
+        Get the widget corresponding to the given key.
+        :param key: The key of the widget to retrieve.
+        :return: The widget object.
+        """
+        return self.widgets_dict[key].wid.value
+    
+    #=====================================================================
+    def __setitem__(self, key, value):
+        """
+        Set the value of the widget corresponding to the given key.
+        :param key: The key of the widget to set.
+        :param value: The value to set the widget to.
+        """
+        if key in self.widgets_dict:
+            wid = self.widgets_dict[key].wid
+            wid.value = value
     
 
 #=====================================================================
-class Masonry(object):
+class Masonry(widgets.Box):
     def __init__(self, forms):
         """
         Create a masonry layout with the provided forms.
@@ -433,14 +458,7 @@ class Masonry(object):
         box : ipywidgets.Box
             The main container widget for the masonry layout.
         """
+        super().__init__()
         self.forms = forms
-        self.box = widgets.Box([form.vbox for form in forms])
-        self.box.add_class('ifk-masonry')
-        
-    #=====================================================================
-    def display(self):
-        """
-        Display the masonry layout in the notebook.
-        """
-        items = [self.box, *load_stylesheets()]
-        display(*items)
\ No newline at end of file
+        self.children = forms
+        self.add_class('ifk-masonry')
\ No newline at end of file
